from typing import Optional, List, Tuple

from shimoku.api.base_resource import Resource
from shimoku.api.resources.business import Business
from shimoku.api.resources.activity_template import ActivityTemplate
from shimoku.api.client import ApiClient

import logging
from shimoku.execution_logger import log_error

logger = logging.getLogger(__name__)


class Universe(Resource):
    _module_logger = logger
    resource_type = "universe"

    def __init__(self, api_client: ApiClient, uuid: str):
        super().__init__(
            api_client=api_client, uuid=uuid, children=[Business, ActivityTemplate]
        )

    async def create_universe_api_key(self, description: str) -> dict:
        endpoint = (
            self._base_resource.base_url + f"universe/{self._base_resource.id}/apiKey"
        )

        params = dict(userType="ADMIN", enabled=True, description=description)

        return await self._base_resource.api_client.query_element(
            method="POST", endpoint=endpoint, **{"body_params": params}
        )

    async def get_universe_api_keys(self) -> list[dict]:
        endpoint = (
            self._base_resource.base_url + f"universe/{self._base_resource.id}/apiKeys"
        )

        return (
            await self._base_resource.api_client.query_element(
                method="GET",
                endpoint=endpoint,
            )
        )["items"]

    async def delete_universe_api_key(self, uuid: str) -> bool:
        endpoint = (
            self._base_resource.base_url
            + f"universe/{self._base_resource.id}/apiKey/{uuid}"
        )

        return await self._base_resource.api_client.query_element(
            method="DELETE",
            endpoint=endpoint,
        )

    # Business methods
    async def create_business(
        self, name: str, theme: Optional[dict] = None
    ) -> Business:
        if self._base_resource.api_client.playground:
            log_error(
                logger, "Cannot create business in local environment", RuntimeError
            )
        return await self._base_resource.create_child(
            Business, alias=name, theme=theme if theme else {}
        )

    async def update_business(
        self, uuid: Optional[str] = None, name: Optional[str] = None, **params
    ) -> bool:
        if params.get("new_name") is not None:
            params["name"] = params.pop("new_name")
            params["new_alias"] = True
        if self._base_resource.api_client.playground:
            uuid, name = "local", None
        return await self._base_resource.update_child(
            Business, uuid=uuid, alias=name, **params
        )

    async def get_business(
        self, uuid: Optional[str] = None, name: Optional[str] = None
    ) -> Optional[Business]:
        if self._base_resource.api_client.playground:
            uuid, name = "local", None
        return await self._base_resource.get_child(Business, uuid, name)

    async def get_businesses(self) -> List[Business]:
        return await self._base_resource.get_children(Business)

    async def delete_business(
        self, uuid: Optional[str] = None, name: Optional[str] = None
    ) -> bool:
        if self._base_resource.api_client.playground:
            log_error(logger, "Cannot delete local business", RuntimeError)
        return await self._base_resource.delete_child(Business, uuid, name)

    # Activity template methods
    async def get_activity_template(
        self, uuid: Optional[str] = None, name_version: Optional[Tuple[str, str]] = None
    ) -> Optional[ActivityTemplate]:
        return await self._base_resource.get_child(ActivityTemplate, uuid, name_version)

    @staticmethod
    def interpret_version(version: str) -> list:
        """Interpret a version string by appending '0' to each section for sorting."""
        numbers = "0123456789"
        if not version:
            return [0, 0, 0]

        def interpret_version_section(section: str) -> list:
            """Interpret a version section, considering non-numeric characters as earlier."""
            res_section = []
            number = ""
            for c in section:
                if c in numbers:
                    number += c
                else:
                    if number:
                        res_section.append(ord("z") + int(number))
                        number = ""
                    res_section.append(ord(c))
            if number:
                res_section.append(ord("z") + int(number))
            return res_section

        return [
            interpret_version_section(section + "0") for section in version.split(".")
        ]

    async def get_activity_templates(self) -> list[ActivityTemplate]:
        templates = await self._base_resource.get_children(ActivityTemplate)
        return sorted(
            templates, key=lambda t: [t["name"]] + self.interpret_version(t["version"])
        )



from shimoku.api.resources.universe import Universe
from shimoku import ApiClient

import logging
from shimoku.execution_logger import ClassWithLogging

logger = logging.getLogger(__name__)


class UniversesLayer(ClassWithLogging):
    """
    Class used to interact with the API at the universe level
    """

    _module_logger = logger
    _use_info_logging = True

    def __init__(self, api_client: ApiClient):
        self._api_client = api_client

    async def create_universe_api_key(self, uuid: str, description: str) -> dict:
        """
        Create a universe API key, this key has admin privileges.
        :param uuid: uuid of the universe
        :param description: description of the key
        """
        universe = Universe(api_client=self._api_client, uuid=uuid)
        return await universe.create_universe_api_key(description)

    async def get_universe_api_keys(self, uuid: str) -> list[dict]:
        """
        Get the universe API keys.
        :param uuid: uuid of the universe
        """
        universe = Universe(api_client=self._api_client, uuid=uuid)
        return await universe.get_universe_api_keys()

    async def delete_universe_api_key(self, uuid: str, api_key_uuid: str) -> bool:
        """
        Delete a universe API key.
        :param uuid: uuid of the universe
        :param api_key_uuid: uuid of the API key
        """
        universe = Universe(api_client=self._api_client, uuid=uuid)
        return await universe.delete_universe_api_key(api_key_uuid)

    async def get_universe_workspaces(self, uuid: str) -> list[dict]:
        """
        Get the universe workspaces.
        :param uuid: uuid of the universe
        """
        universe = Universe(api_client=self._api_client, uuid=uuid)
        return [b.cascade_to_dict() for b in await universe.get_businesses()]

    async def get_universe_activity_templates(self, uuid: str) -> list[dict]:
        """
        Get the universe activity templates as a list of dictionaries.
        :param uuid: uuid of the universe
        """
        universe = Universe(api_client=self._api_client, uuid=uuid)
        return [
            {
                "name": at["name"],
                "description": at["description"],
                "min_run_interval": at["minRunInterval"],
                "input_settings": [
                    {
                        "name": name,
                        "description": param["description"],
                        "datatype": param["datatype"],
                    }
                    for name, param in at["inputSettings"].items()
                    if isinstance(param, dict)
                ],
            }
            for at in await universe.get_activity_templates()
            if at["enabled"]
        ]



from shimoku.actions.execute_action import execute_action
from shimoku.exceptions import ActionError
from os import path, listdir
import unittest

correct_scripts_path = path.join(path.dirname(__file__), "correct_action_scripts")
error_scripts_path = path.join(path.dirname(__file__), "error_action_scripts")


class TestActions(unittest.TestCase):
    def test_action(self):
        for correct_action_script in listdir(correct_scripts_path):
            execute_action(
                open(path.join(correct_scripts_path, correct_action_script)).read(),
                True,
            )

        for code_with_errors in listdir(error_scripts_path):
            print()
            with self.assertRaises(ActionError):
                execute_action(code_with_errors, True)



import inspect
import importlib
import os
from shimoku.utils import get_args_with_defaults

base_path = os.path.abspath(os.path.join(os.path.dirname(__file__), ".."))
target_paths = {
    os.path.join(base_path, "src", "shimoku", "api", "user_access_classes"): "*",
    os.path.join(base_path, "src", "shimoku", "ai"): "ai_layer.py",
    os.path.join(base_path, "src", "shimoku", "plt"): "plt_layer.py",
}


def handle_annotation(annotation: type) -> str:
    if hasattr(annotation, "_use_info_logging") and annotation._use_info_logging:
        return f"{annotation.__name__}Header"
    if annotation is type(None) or annotation is None:
        type_str = "None"
    elif hasattr(annotation, "__name__"):
        type_str = annotation.__name__
    else:
        type_str = str(annotation)
    return (
        type_str.replace("NoneType", "None")
        .replace("pandas.core.frame.DataFrame", " DataFrame")
        .replace("typing.", "")
        .replace("<built-in function callable>", "callable")
    )


def handle_parameter(name: str, annotation: type, default_args: dict) -> str:
    if name in default_args:
        default_value = default_args[name]
        if isinstance(default_value, str):
            default_value = f'"{default_value}"'
        return f"        {name}: {handle_annotation(annotation)} = {default_value},"
    else:
        return f"        {name}: {handle_annotation(annotation)},"


for target_path, target_file in target_paths.items():
    output_path = os.path.join(target_path, "generated_headers")
    if not os.path.exists(output_path):
        os.makedirs(output_path)
    for file in os.listdir(target_path):
        if file != target_file and target_file != "*":
            continue
        if file.endswith(".py") and not file.startswith("__"):
            print(
                f'{os.path.relpath(target_path, base_path).replace(os.sep, ".")[4:]}.{file[:-3]}'
            )
            module = importlib.import_module(
                f'{os.path.relpath(target_path, base_path).replace(os.sep, ".")[4:]}.{file[:-3]}'
            )
            classes_for_generation = []
            for name, obj in inspect.getmembers(module):
                if not inspect.isclass(obj):
                    continue
                if name.startswith("_"):
                    continue
                if not hasattr(obj, "_use_info_logging") or not obj._use_info_logging:
                    continue
                classes_for_generation.append((name, obj))

            if not classes_for_generation:
                continue

            classes_code_lines = []
            for class_for_generation in classes_for_generation:
                print(class_for_generation[0])
                methods = inspect.getmembers(
                    class_for_generation[1], predicate=inspect.isfunction
                )
                methods_code_lines = []
                for method in methods:
                    if method[0].startswith("_"):
                        continue
                    methods_code_lines.extend(
                        [
                            f"    def {method[0]}(",
                            "        self,",
                            *[
                                handle_parameter(
                                    arg,
                                    method[1].__annotations__[arg],
                                    get_args_with_defaults(method[1]),
                                )
                                for arg in method[1].__annotations__
                                if arg not in ["self", "return"]
                            ],
                        ]
                    )
                    sig = inspect.signature(method[1])
                    params = sig.parameters.values()
                    has_kwargs = any([True for p in params if p.kind == p.VAR_KEYWORD])
                    if has_kwargs:
                        methods_code_lines.append("        **kwargs,")
                    if "return" in method[1].__annotations__:
                        methods_code_lines.append(
                            f'    ) -> {handle_annotation(method[1].__annotations__["return"])}:'
                        )
                    else:
                        methods_code_lines.append("    ):")
                    methods_code_lines.extend(
                        [
                            f'        """{method[1].__doc__}"""',
                            "        pass",
                            "",
                        ]
                    )

                class_code_lines = [
                    "",
                    f"class {class_for_generation[0]}Header:",
                    f'    """{class_for_generation[1].__doc__}"""',
                    "",
                    *methods_code_lines,
                ]
                classes_code_lines.extend(class_code_lines)
            main_code_lines = [
                "# This file was generated automatically by scripts/user_classes_header_generator.py do not modify it!",
                "# If the user access files are modified, this file has to be regenerated with the script.",
                "from typing import Optional, Union",
                "from pandas import DataFrame",
                "",
                *classes_code_lines,
            ]
            with open(
                os.path.join(output_path, f"{classes_for_generation[-1][0]}Header.py"),
                "w",
            ) as f:
                f.write("\n".join(main_code_lines))
